// HookTest.cpp : Defines the entry point for the application.
//

#define INITGUID

#include <windows.h>
#include <windowsx.h>
#include <aygshell.h>
#include <shellapi.h>
#include <aygshell.h>
#include "resource.h"
#include <pimstore.h>


#define ITEM_TESTMENU 99100
#define WM_CONTACTS_TESTHOOK WM_USER + 0x300


HWND contactsWindow;
HWND listWindow;
WNDPROC oldContactsWindowProc;
IPOutlookItemCollection *outlookContacts = NULL;
int maxContacts;
BOOL SIMContactsReady = FALSE;
	
LRESULT CALLBACK hookedContactsWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, 
											LPARAM lParam)
{
	if (uMsg == WM_CONTACTS_TESTHOOK) {
		BOOL *hookTest = (BOOL*)lParam;
		*hookTest = TRUE;
		return 0;
	}

	if (uMsg == WM_COMMAND && wParam == ITEM_TESTMENU) {
		IContact *pContact = NULL;
		int mark = ListView_GetNextItem(listWindow, 0xffffffff, LVNI_SELECTED);
		BSTR firstName;
		BSTR lastName;
		TCHAR info[100];

		if (mark + 1 > maxContacts) {
			MessageBox(contactsWindow, TEXT("SIM contacts not available"), TEXT("Hook test"), MB_OK);
			return 0;
		}

		outlookContacts->Item (mark + 1, (IDispatch**)&pContact);
		pContact->get_FirstName(&firstName);
		pContact->get_LastName(&lastName);
		wsprintf(info, TEXT("Contact selected : %s %s"), firstName, lastName);
		MessageBox(contactsWindow, info, TEXT("Hook test"), MB_OK);
		SysFreeString(firstName);
		SysFreeString(lastName);

		return 0;
	}

	return CallWindowProc(oldContactsWindowProc, hwnd, uMsg, wParam, lParam);
}

// -------------------------------------------------------------------------------

#define CONTACTS_CLASS_NAME "MSContacts"

BOOL WINAPI CALLBACK checkContactsLoadedCallback(HWND handle, LPARAM param) {
	HWND *result = (HWND*)param;
	TCHAR className[100];

	GetClassName(handle, className, sizeof(className) / sizeof(TCHAR));
	
	if (wcscmp(className, TEXT(CONTACTS_CLASS_NAME)) == 0) {
		*result = handle;
		return FALSE;
	}

	return TRUE;
}


BOOL checkContactsLoaded(HWND *result) {

	*result = 0;

	EnumWindows(checkContactsLoadedCallback, (LPARAM)result);

	return (*result != 0);
}

// -------------------------------------------------------------------------------

#define TESTMENU_TEXT "Test hook"

void hookWindowMenu(HWND contactsWindow) {
	HWND contactsMenuBar;
	HMENU mainMenu;
	MENUITEMINFO itemInfo;

	// Get the menubar (window)

	contactsMenuBar = SHFindMenuBar(contactsWindow);
	
	// Get the second menu (menu)

	mainMenu = (HMENU)SendMessage(contactsMenuBar, SHCMBM_GETMENU, 0, 0);
	memset(&itemInfo, 0, sizeof(itemInfo));
	itemInfo.cbSize = sizeof(itemInfo);
	itemInfo.fMask = MIIM_ID | MIIM_SUBMENU;
	GetMenuItemInfo(mainMenu, 0, TRUE, &itemInfo);

	mainMenu = itemInfo.hSubMenu;

	InsertMenu(mainMenu, 0xffffffff, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
	InsertMenu(mainMenu, 0xffffffff, MF_BYPOSITION | MF_STRING, ITEM_TESTMENU, TEXT(TESTMENU_TEXT));		
}

void hookWindowProcessing(HWND contactsWindow) {
	oldContactsWindowProc = (WNDPROC)GetWindowLong(contactsWindow, GWL_WNDPROC);
	SetWindowLong(contactsWindow, GWL_WNDPROC, (LONG)hookedContactsWindowProc);
}

// -------------------------------------------------------------------------------

void setupPOContacts(HWND contactsWindow, IPOutlookApp **outlookApp, IFolder **outlookContactsFolder) {
	HRESULT result;

	result = CoCreateInstance(CLSID_Application, NULL, CLSCTX_INPROC_SERVER, IID_IPOutlookApp, (LPVOID *)outlookApp);
	if (FAILED(result)) 
		return;
	result = (*outlookApp)->Logon((long)contactsWindow);
	if (FAILED(result))
		return;
	result = (*outlookApp)->GetDefaultFolder (olFolderContacts, outlookContactsFolder);
	if (FAILED(result))
		return;
	result = (*outlookContactsFolder)->get_Items(&outlookContacts);
	if (FAILED(result))
		return;
	outlookContacts->get_Count(&maxContacts);
}

// -------------------------------------------------------------------------------

#define CONTACTS_PATH "\\IPSM\\Windows\\Start Menu\\Contacts.lnk"

int WINAPI WinMain(
    HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPWSTR lpCmdLine,
    int nCmdShow
    )
{
	HWND contactsWindow;
	HPROCESS contactsProcess;
	IPOutlookApp *outlookApp;
	IFolder *outlookContactsFolder;

	// Look if Contacts are already loaded

	if (checkContactsLoaded(&contactsWindow)) {
		BOOL hookTest = FALSE;

		// Check if it was already hooked
		SendMessage(contactsWindow, WM_CONTACTS_TESTHOOK, 0, (LPARAM)&hookTest);
		if (hookTest) 
			return 0;		// let the other instance deal with it
		GetWindowThreadProcessId(contactsWindow, (LPDWORD)&contactsProcess);
	}
	else {
		// Need to start Contacts app

		SHELLEXECUTEINFO shellExecuteInfo;
		memset(&shellExecuteInfo, 0, sizeof(shellExecuteInfo));
		shellExecuteInfo.cbSize = sizeof(shellExecuteInfo);
		shellExecuteInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
		shellExecuteInfo.hwnd = NULL;
		shellExecuteInfo.lpVerb = TEXT("open");
		shellExecuteInfo.lpFile = TEXT(CONTACTS_PATH);
		shellExecuteInfo.nShow = SW_SHOW;
		if (!ShellExecuteEx(&shellExecuteInfo))
			return 0;   // couldn't start Contacts, nothing to do ...
		contactsProcess = shellExecuteInfo.hProcess;

		// Get the Contacts Window handle
		for (;;) {
			TCHAR className[100];

			contactsWindow = GetForegroundWindow();
			GetClassName(contactsWindow, className, sizeof(className) / sizeof(TCHAR));
			if (wcscmp(className, TEXT(CONTACTS_CLASS_NAME)) == 0)
				break;
			Sleep(500);
		}
	}

	// Get List Window handle
	listWindow = GetWindow(contactsWindow, GW_CHILD);
	// Hook our menu
	hookWindowMenu(contactsWindow);
	// Hook the processing
	hookWindowProcessing(contactsWindow); 
	// Set up Pocket Outlook contacts
	setupPOContacts(contactsWindow, &outlookApp, &outlookContactsFolder);

	// Set up SIM contacts
	//setupSIMContacts();

	// All systems go ... now wait Wait wait ...
	while (WaitForSingleObject((void*)contactsProcess, 200) != WAIT_OBJECT_0) 
		Sleep(500);

	// Garbage collect and finish
	outlookContactsFolder->Release();
	outlookApp->Release();

    return 0;
}
